/* ----------------------------------------------------------
   validation.js - Provides a set of basic input validations
   to be used on text form fields. Uses regular expressions.
   See http://www.regular-expressions.info/reference.html. 
   ---------------------------------------------------------- */ 

/*--------------  BLANK HANDLING FUNCTIONS -----------------*/
function trim(inpStr) {
	// Returns inpStr with any leading 
	// or trailing whitespace characters removed
	regex = /^\s+|\s+$/g
	return inpStr.replace(regex,'');
}

function isBlank( inpStr ) {
	// Returns true if inpStr contains 
	// only whitespace characters
	return (trim(inpStr) == '');
}

/*--------------  SPECIAL DATA TYPE FUNCTIONS -----------------*/
function isSSN( inpStr ) {
	// Returns true if inpStr is a SNN in one of these formats: 
	// 999-99-9999, 999 99 9999, 999999999
	inpStr = trim(inpStr);
	regex1 = /^\d{3}-\d{2}-\d{4}$/
	regex2 = /^\d{3} \d{2} \d{4}$/
	regex3 = /^\d{9}$/
	return regex1.test(inpStr) || regex2.test(inpStr) || regex3.test(inpStr);
}

function isPhoneNum( inpStr ) {
	// Returns true if inpStr is a Phone# 
	// in the format of (999)999-9999
	inpStr = trim(inpStr);
	regex = /^\(\d{3}\)\d{3}-\d{4}$/g
	return regex.test(inpStr);
}

/*--------------  NUMERIC DATA HANDLING FUNCTIONS -----------------*/
function isInteger( inpStr ) {
	// Returns true if inpStr is an integer 
	inpStr = trim(inpStr);
	regex = /^\d+$/
	return regex.test(inpStr);
}

function isFloat( inpStr ) {
	// Returns true if inpStr is a floating point number 
	inpStr = trim(inpStr);
	regex = /^\d+\.\d+$/
	return regex.test(inpStr);
}

function isAmount( inpStr ) {
	// Returns true if inpStr is a dollar amount 
	// i.e. a number with two decimal places. 
	inpStr = trim(inpStr);
	regex = /^\d+\.\d{2}$/
	return regex.test(inpStr);
}
   
/*----------------  DATE HANDLING FUNCTIONS -----------------*/
function formatDateMDCY( inpStr ) {
	// If inpStr is an 8 digit string, returns the input
	// as though a Y edit code was used (99/99/9999).
	// Else, returns the input unchanged.
	inpStr = trim(inpStr);
	var regex = /^\d{8}$/ 
	if (regex.test(inpStr)) {
	   	var returnVal = inpStr.substr(0,2) + '/' +
      				inpStr.substr(2,2) + '/' +
      				inpStr.substr(4,4) 
   		return returnVal; 
	} else {
		return inpStr;
	}
}

function isDateFormat( inpStr ) {
	// Returns true if inpStr is in format 99/99/9999
	var regex = /^\d{2}\/\d{2}\/\d{4}$/ 
	return regex.test(inpStr);
}

function isValidDate( inpStr ) {
   // Returns true if inpStr is a valid date in format mm/dd/ccyy
   // Performs check for valid month, day and year ranges using this method:
   // 1. Split out month, day, and full year from the input,
   // 2. Create a date object from these 3 components,
   // 3. Then extract month, day, and full year from the date object.
   // If they match what we started with, date was valid. 
   
   // 1. Split out month, day, and full year from the input,
   var mdy = inpStr.split("/");
   var monthfield = mdy[0];
   var dayfield = mdy[1];
   var yearfield = mdy[2];
   // 2. Create a date object from these 3 components,
   var dayobj = new Date(yearfield, monthfield-1, dayfield);
   // alert("Date is " + dayobj); // Debug code
   // 3. Then extract month, day, and full year from the date object.
   // If they match what we started with, date was valid. 
   if ((dayobj.getMonth()+1  == monthfield)
   &&  (dayobj.getDate()     == dayfield)
   &&  (dayobj.getFullYear() == yearfield))
   	return true;
   else
   	return false;
}
   