<?php
/* Copyright (C) 2016 John Valance - All Rights Reserved.
   You may use, distribute and modify this code under the
   terms of the BSD license. See file LICENSE.txt in the
   top-most parent folder distributed with this source code.
   If missing, please write to: johnv@div1sys.com.
*/
require_once('../03-singleton/DB2Connection.php');

class Person_AR {

    private $id;
    private $firstName;
    private $lastName;
    private $dateOfBirth;

    public function __construct( $id = null )
    {
        if ( isset($id) ) {
            $this->find($id);
        }
    }

    /*
     * Static factory method - create a Person_AR object from DB data
     */
    public static function load(array $data) {
        $person = new Person_AR;
        $person->id = $data['ID'];
        $person->setFirstName($data['FIRSTNAME']);
        $person->setLastName($data['LASTNAME']);
        $person->setDateOfBirth($data['DATEOFBIRTH']);
        return $person;
    }
    /*
     * Getters and Setters
     */
    public function getId() { return $this->id; }

    public function getFirstName() { return $this->firstName; }
    public function getLastName() { return $this->lastName; }
    public function getDateOfBirth() { return $this->dateOfBirth; }

    public function setFirstName($firstName) { $this->firstName = trim($firstName); }
    public function setLastName($lastName) { $this->lastName = trim($lastName); }

    public function setDateOfBirth($dateOfBirth) {
        $dateVar = strtotime($dateOfBirth);
        $dateYMD = date('Y-m-d', $dateVar);

        if ($dateYMD !== $dateOfBirth) {
            throw new Exception("Date of Birth is not a valid date format: '{$dateOfBirth}'");
        } else {
            $this->dateOfBirth = $dateOfBirth;
        }
    }

    public static function delete_all( ) {
        $conn = DB2Connection::getInstance();
        if (!db2_exec($conn, "delete from person"))
            throw new Exception("Delete from person failed! : ". db2_stmt_errormsg());
    }

    private function validateData() {
        static $emptyMsg = 'is required for insert or update';

        if ( empty($this->getFirstName() ) )
            throw new Exception("firstName $emptyMsg");

        if ( empty($this->getLastName() ) )
            throw new Exception("lastName $emptyMsg");

        if (  empty($this->getDateOfBirth() ) )
            throw new Exception("dateOfBirth $emptyMsg");

        return true;
    }

    /*
     * FINDER METHODS
     */
    public function find( $id ) {
        if (!empty($this->id) && ($id != $this->id) ) {
            // We already retrieved or saved this record.
            // Do not allow changing the identity of this object
            throw new Exception("Person ID already set to {$this->getId()}");
        }
        $conn = DB2Connection::getInstance();

        $select_row_sql = "select * from person where id = ?";

        $select_row_stmt = db2_prepare($conn, $select_row_sql);
        if (!$select_row_stmt) throw new Exception("Select ROW prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($select_row_stmt, array($id)))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());

        $row = db2_fetch_assoc($select_row_stmt);

        if (!is_array($row)) {
            throw new Exception("Person record not found for id = $id");
        } else {
            $this->id = $row['ID'];
            $this->setFirstName($row['FIRSTNAME']);
            $this->setLastName($row['LASTNAME']);
            $this->setDateOfBirth($row['DATEOFBIRTH']);
        }

        return $row;
    }


    public static function find_all() {
        $conn = DB2Connection::getInstance();

        $sql = "select * from person";

        $stmt = db2_prepare($conn, $sql);
        if (!$stmt) throw new Exception("Select prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($stmt))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());;

        $obj_list = array();
        while ($row = db2_fetch_assoc($stmt )) {
            $obj_list[] = Person_AR::load($row);
        }

        return $obj_list;
    }

    public static function findByLastName( $lastName ) {
        $conn = DB2Connection::getInstance();

        $sql = "select * from person where upper(lastName) like upper('%?%')";

        $stmt = db2_prepare($conn, $sql);
        if (!$stmt) throw new Exception("Select prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($stmt, array($lastName)))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());;

        $obj_list = array();
        while ($row = db2_fetch_assoc($stmt )) {
            $obj_list[] = Person_AR::load($row);
        }

        return $obj_list;
    }

    /*
     * Database updating methods
     */
    public function save() {
        if (!empty($this->id)) {
            $this->update();
        } else {
            $this->insert();
        }
    }

    public function insert( ) {
        $this->validateData();
        $conn = DB2Connection::getInstance();

        $insert_sql =
            "insert into person (firstName, lastName, dateOfBirth) values (?, ?, ?)";
        $bindVals = array($this->firstName, $this->lastName, $this->dateOfBirth );

        $insert_stmt = db2_prepare($conn, $insert_sql);
        if (!$insert_stmt) throw new Exception("Prepare INSERT failed! : ". db2_stmt_errormsg());

        if (!db2_execute($insert_stmt, $bindVals))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());
        $this->id = db2_last_insert_id($conn);
        return $this->id;
    }

    public function update( ) {
        $this->validateData();
        $conn = DB2Connection::getInstance();

        $update_sql =
            "update person set firstName = ?, lastName = ?, dateOfBirth = ? where id = ?";
        $bindVals = array($this->firstName, $this->lastName, $this->dateOfBirth, $this->id );

        $update_stmt = db2_prepare($conn, $update_sql);
        if (!$update_stmt) throw new Exception("Prepare UPDATE failed! : ". db2_stmt_errormsg());

        if (!db2_execute($update_stmt, $bindVals))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());

        return true;
    }

    public function delete( ) {
        $conn = DB2Connection::getInstance();

        $delete_row_sql = "delete from person where id = ?";

        $delete_row_stmt = db2_prepare($conn, $delete_row_sql);
        if (!$delete_row_stmt) throw new Exception("Delete row prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($delete_row_stmt, array($this->id)))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());

        // Unset all properties of this object
        foreach ($this as $key => $value) {
            unset($this->$key);
        }
    }

    /*
     * DOMAIN LOGIC (i.e. business logic) METHODS
     */
    public function getFullName( ) {
        return $this->getFirstName() . ' ' . $this->getLastName();
    }

    public function getDateOfBirthFormatted( ) {
        return date('D M d, Y', strtotime($this->dateOfBirth));
    }

    public function getAge( ) {
        $datetime1 = new DateTime(date('Ymd'));
        $datetime2 = new DateTime($this->dateOfBirth);
        $interval = $datetime2->diff($datetime1);
        return $interval->format('%r%y years, %m months, %d days');
    }

    public function getAgeText( ) {
        $currDate = date('M d, Y');
        return "{$this->getFullName()} was born on {$this->getDateOfBirthFormatted()}, "
        . "and {$this->getFirstName()}'s age as of $currDate is {$this->getAge()}";

    }

    public function __toString() {
        return <<<PERSON_STRING
    First Name: {$this->getFirstName()}
    Last Name: {$this->getLastName()}
    Date of Birth: {$this->getDateOfBirth()}
PERSON_STRING;
    }

}
